
# This file is automatically generated during the generation of setup.py
# Copyright 2020, Caer

author = 'Jason Dsouza: http://www.github.com/jasmcaus'
version = '1.5.1'
full_version = '1.5.1'
release = True
if not release:
    version = full_version
